<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:plm="http://www.plmxml.org/Schemas/PLMXMLSchema"
	xmlns:cim="http://www.coolemall.eu/DEBBComponent"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<!-- Specification of the output document -->
	<xsl:output method="xml" version="1.0" encoding="UTF-8"
		indent="yes" />
	<xsl:template match="/">
		<xsl:element name="environment">
			<xsl:attribute name="xsi:noNamespaceSchemaLocation">DCWormsResSchema.xsd</xsl:attribute>
			<xsl:element name="resources">
				<xsl:element name="scheduler"></xsl:element>
				<xsl:apply-templates select="//*[ComponentId]" />
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template match="//*[ComponentId]">
		<xsl:element name="computingResource">
			<xsl:attribute name="class"> 
		 		<xsl:value-of select="name(.)" /> 
		 	</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="ComponentId" />
			</xsl:attribute>
			<xsl:if test="Memory">
				<xsl:apply-templates select="Memory" />
			</xsl:if>
			<xsl:if test="Storage">
				<xsl:apply-templates select="Storage" />
			</xsl:if>
			<xsl:if test="CoolingDevice">
				<xsl:apply-templates select="CoolingDevice" />
			</xsl:if>
			<xsl:call-template name="BasicDEBBPhysicalElementData" />
			<xsl:if test="Baseboard">
				<xsl:apply-templates select="Baseboard" />
			</xsl:if>
			<xsl:if test="Processor">
				<xsl:apply-templates select="Processor" />
			</xsl:if>
		</xsl:element>
	</xsl:template>

	<xsl:template match="Baseboard">
		<xsl:element name="parameter">
			<xsl:attribute name="name">Baseboard</xsl:attribute>
			<!-- Don't call BasicDEBBPhysicalElementData, because parameter cannot 
				contain parameters. Must be parameter containing properties. -->
			<!-- <xsl:call-template name="BasicDEBBPhysicalElementData" /> -->
			<xsl:if test="Manufacturer">
				<xsl:element name="property">
					<xsl:attribute name="name">Manufacturer</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="Manufacturer" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
			<xsl:if test="Product">
				<xsl:element name="property">
					<xsl:attribute name="name">Product</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="Product" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
			<xsl:if test="MaxPower">
				<xsl:element name="property">
					<xsl:attribute name="name">MaxPower</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="MaxPower" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
		</xsl:element>

	</xsl:template>

	<xsl:template match="Processor">
		<xsl:element name="computingResource">
			<xsl:attribute name="class">Processor</xsl:attribute>
			<xsl:call-template name="BasicDEBBPhysicalElementData" />
			<xsl:element name="parameter">
				<xsl:attribute name="name">MaxClockSpeed</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="MaxClockSpeed" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="profile">
				<xsl:comment>
					Power profile and/or air throughput
				</xsl:comment>
				<xsl:if test="PState | CState">
					<!-- If PState or CState elements present add powerProfile element -->
					<xsl:element name="powerProfile">
						<xsl:element name="powerStates">
							<xsl:apply-templates select="PState" />
						</xsl:element>
					</xsl:element>
				</xsl:if>
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template match="PState">
		<xsl:element name="powerState">
			<xsl:element name="name">
				<xsl:value-of select="State" />
			</xsl:element>
			<xsl:element name="powerUsage">
				<xsl:value-of select="PowerUsage" />
			</xsl:element>
			<xsl:element name="transition">
				<xsl:element name="to">
					<xsl:value-of select="Transition/ToState" />
				</xsl:element>
				<xsl:element name="powerUsage">
					<xsl:value-of select="Transition/PowerUsage" />
				</xsl:element>
				<xsl:element name="time">
					<xsl:value-of select="Transition/Time" />
				</xsl:element>
			</xsl:element>
			<!-- Voltage & Frequency -->
			<xsl:element name="parameter">
				<xsl:attribute name="name">Voltage</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Voltage" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="parameter">
				<xsl:attribute name="name">Frequency</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Frequency" />
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template match="Memory">
		<xsl:element name="resourceUnit">
			<xsl:attribute name="class">Memory</xsl:attribute>
			<xsl:element name="amount">
				<xsl:value-of select="Capacity" />
			</xsl:element>
			<xsl:call-template name="BasicDEBBPhysicalElementData" />
		</xsl:element>
	</xsl:template>

	<xsl:template match="Storage">
		<!-- TODO: Not finished -->
		<xsl:element name="resourceUnit">
			<xsl:attribute name="class">Storage</xsl:attribute>
			<xsl:element name="amount">
				<xsl:attribute name="unit">MB</xsl:attribute>
				1
			</xsl:element>
			<!-- <xsl:call-template name="BasicDEBBPhysicalElementData" /> -->
		</xsl:element>
	</xsl:template>

	<xsl:template match="CoolingDevice">
		<!-- TODO: Not finished -->
		<xsl:element name="profile">
			<xsl:element name="airThroughputProfile">
				<!-- <xsl:attribute name="name">CoolingDevice</xsl:attribute> -->
				<!-- TODO: What to take it from?? -->
<!--				<xsl:element name="airThroughputEstimationPlugin">-->
<!--				</xsl:element>				-->
				<xsl:if test="ThroughputProfile">
					<xsl:element name="airThroughputStates">
						<xsl:apply-templates select="ThroughputProfile" />
					</xsl:element>
				</xsl:if>
				<!-- Obligatory attribute -->
				<xsl:element name="parameter">
					<xsl:attribute name="name">Class</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="Class" />
					</xsl:element>
				</xsl:element>
				<xsl:if test="MaxCoolingCapacity">
					<xsl:element name="parameter">
						<xsl:attribute name="name">MaxCoolingCapacity</xsl:attribute>
						<xsl:element name="value">
							<xsl:value-of select="MaxCoolingCapacity" />
						</xsl:element>
					</xsl:element>
				</xsl:if>
				<xsl:if test="MaxAirThroughput">
					<xsl:element name="parameter">
						<xsl:attribute name="name">MaxAirThroughput</xsl:attribute>
						<xsl:element name="value">
							<xsl:value-of select="MaxAirThroughput" />
						</xsl:element>
					</xsl:element>
				</xsl:if>
				<xsl:if test="MaxWaterThroughput">
					<xsl:element name="parameter">
						<xsl:attribute name="name">MaxWaterThroughput</xsl:attribute>
						<xsl:element name="value">
							<xsl:value-of select="MaxWaterThroughput" />
						</xsl:element>
					</xsl:element>
				</xsl:if>				
				<xsl:call-template name="BasicDEBBPhysicalElementData" />
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template match="ThroughputProfile">
		<xsl:apply-templates select="FlowState" />
	</xsl:template>

	<xsl:template match="FlowState">
		<xsl:element name="airThroughputState">
			<xsl:element name="name">
				<xsl:value-of select="State" />
			</xsl:element>
			<xsl:if test="Flow">
				<xsl:element name="value">
					<xsl:value-of select="Flow" />
				</xsl:element>
			</xsl:if>
			<xsl:if test="PowerUsage">
				<xsl:element name="powerUsage">
					<xsl:value-of select="PowerUsage" />
				</xsl:element>
			</xsl:if>
			<xsl:if test="Transition">
				<xsl:apply-templates select="Transition" />
			</xsl:if>
		</xsl:element>
	</xsl:template>

	<xsl:template match="Transition">
		<xsl:element name="parameter">
			<xsl:attribute name="name">ToState</xsl:attribute>
			<xsl:element name="value">
				<xsl:value-of select="ToState" />
			</xsl:element>
		</xsl:element>
		<xsl:element name="parameter">
			<xsl:attribute name="name">PowerUsage</xsl:attribute>
			<xsl:element name="value">
				<xsl:value-of select="PowerUsage" />
			</xsl:element>
		</xsl:element>
		<xsl:element name="parameter">
			<xsl:attribute name="name">Time</xsl:attribute>
			<xsl:element name="value">
				<xsl:value-of select="Time" />
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<!-- Template for transforming basic parameters of all devices of any type -->
	<xsl:template name="BasicDEBBPhysicalElementData">
		<xsl:if test="Manufacturer">
			<xsl:element name="parameter">
				<xsl:attribute name="name">Manufacturer</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Manufacturer" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		<xsl:if test="Product">
			<xsl:element name="parameter">
				<xsl:attribute name="name">Product</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Product" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		<xsl:if test="MaxPower">
			<xsl:element name="parameter">
				<xsl:attribute name="name">MaxPower</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="MaxPower" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>